Step

Enum Step 

pub enum Step<Y, D> {
    Yielded(Y),
    Complete(D),
}
Result of a computation step, either yielding a value to continue or completing with a final value.

Step is the return type for coroutine computations, similar to how Option represents optional values and Result represents fallible operations.

Examples

use sans::Step;

let continuing: Step<i32, String> = Step::Yielded(42);
let completed: Step<i32, String> = Step::Complete("finished".to_string());

// Using combinators
let doubled = continuing.map_yielded(|x| x * 2);
assert_eq!(doubled, Step::Yielded(84));

Variants



Yielded(Y)

Continue computation with an intermediate yield value



Complete(D)

Complete computation with a final value

Implementations

Source

impl<Y, D> Step<Y, D>

pub const fn is_yielded(&self) -> bool

Returns true if the step is Yielded.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Yielded(42);
assert!(x.is_yielded());

let y: Step<i32, &str> = Step::Complete("complete");
assert!(!y.is_yielded());
pub const fn is_complete(&self) -> bool

Returns true if the step is Complete.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Complete("complete");
assert!(x.is_complete());

let y: Step<i32, &str> = Step::Yielded(42);
assert!(!y.is_complete());
pub fn yielded_value(self) -> Option<Y>

Converts from Step<Y, D> to Option<Y>.

Converts self into an Option<Y>, consuming self, and discarding the complete value, if any.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Yielded(42);
assert_eq!(x.yielded_value(), Some(42));

let y: Step<i32, &str> = Step::Complete("complete");
assert_eq!(y.yielded_value(), None);
pub fn complete_value(self) -> Option<D>

Converts from Step<Y, D> to Option<D>.

Converts self into an Option<D>, consuming self, and discarding the yield value, if any.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Complete("complete");
assert_eq!(x.complete_value(), Some("complete"));

let y: Step<i32, &str> = Step::Yielded(42);
assert_eq!(y.complete_value(), None);
pub fn map_complete<D2, F>(self, f: F) -> Step<Y, D2>
where F: FnOnce(D) -> D2,

Maps a Step<Y, D> to Step<Y, D2> by applying a function to the complete value.

Examples
use sans::Step;

let x: Step<i32, i32> = Step::Complete(5);
assert_eq!(x.map_complete(|v| v * 2), Step::Complete(10));

let y: Step<i32, i32> = Step::Yielded(3);
assert_eq!(y.map_complete(|v| v * 2), Step::Yielded(3));
pub fn map_yielded<Y2, F>(self, f: F) -> Step<Y2, D>
where F: FnOnce(Y) -> Y2,

Maps a Step<Y, D> to Step<Y2, D> by applying a function to the yielded value.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Yielded(42);
assert_eq!(x.map_yielded(|v| v * 2), Step::Yielded(84));

let y: Step<i32, &str> = Step::Complete("complete");
assert_eq!(y.map_yielded(|v: i32| v * 2), Step::Complete("complete"));
pub fn map<Y2, D2, FY, FD>(self, fy: FY, fd: FD) -> Step<Y2, D2>
where FY: FnOnce(Y) -> Y2, FD: FnOnce(D) -> D2,

Maps a Step<Y, D> to Step<Y2, D2> by applying functions to both values.

Examples
use sans::Step;

let x: Step<i32, i32> = Step::Yielded(42);
assert_eq!(x.map(|y| y * 2, |d| d + 1), Step::Yielded(84));

let y: Step<i32, i32> = Step::Complete(10);
assert_eq!(y.map(|y| y * 2, |d| d + 1), Step::Complete(11));
pub fn yielded_or(self, default: Y) -> Y

Returns the yielded value or a default.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Yielded(42);
assert_eq!(x.yielded_or(0), 42);

let y: Step<i32, &str> = Step::Complete("complete");
assert_eq!(y.yielded_or(0), 0);
pub fn yielded_or_else<F>(self, f: F) -> Y
where F: FnOnce() -> Y,

Returns the yielded value or computes it from a closure.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Yielded(42);
assert_eq!(x.yielded_or_else(|| 0), 42);

let y: Step<i32, &str> = Step::Complete("complete");
assert_eq!(y.yielded_or_else(|| 0), 0);
pub fn complete_or(self, default: D) -> D

Returns the complete value or a default.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Complete("complete");
assert_eq!(x.complete_or("default"), "complete");

let y: Step<i32, &str> = Step::Yielded(42);
assert_eq!(y.complete_or("default"), "default");
pub fn complete_or_else<F>(self, f: F) -> D
where F: FnOnce() -> D,

Returns the complete value or computes it from a closure.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Complete("complete");
assert_eq!(x.complete_or_else(|| "default"), "complete");

let y: Step<i32, &str> = Step::Yielded(42);
assert_eq!(y.complete_or_else(|| "default"), "default");
pub const fn as_ref(&self) -> Step<&Y, &D>

Converts from &Step<Y, D> to Step<&Y, &D>.

Examples
use sans::Step;

let x: Step<i32, String> = Step::Yielded(42);
assert_eq!(x.as_ref(), Step::Yielded(&42));

let y: Step<i32, String> = Step::Complete("complete".to_string());
assert_eq!(y.as_ref(), Step::Complete(&"complete".to_string()));
pub fn as_mut(&mut self) -> Step<&mut Y, &mut D>

Converts from &mut Step<Y, D> to Step<&mut Y, &mut D>.

Examples
use sans::Step;

let mut x: Step<i32, String> = Step::Yielded(42);
if let Step::Yielded(y) = x.as_mut() {
    *y = 100;
}
assert_eq!(x, Step::Yielded(100));
pub fn flip(self) -> Step<D, Y>

Converts from Step<Y, D> to Step<D, Y> by swapping variants.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Yielded(42);
assert_eq!(x.flip(), Step::Complete(42));

let y: Step<i32, &str> = Step::Complete("complete");
assert_eq!(y.flip(), Step::Yielded("complete"));
pub fn contains_yielded<U>(&self, y: &U) -> bool
where U: PartialEq<Y>,

Returns true if the step is a Yielded value containing the given value.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Yielded(42);
assert!(x.contains_yielded(&42));
assert!(!x.contains_yielded(&100));

let y: Step<i32, &str> = Step::Complete("complete");
assert!(!y.contains_yielded(&42));
pub fn contains_complete<U>(&self, d: &U) -> bool
where U: PartialEq<D>,

Returns true if the step is a Complete value containing the given value.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Complete("complete");
assert!(x.contains_complete(&"complete"));
assert!(!x.contains_complete(&"other"));

let y: Step<i32, &str> = Step::Yielded(42);
assert!(!y.contains_complete(&"complete"));
pub fn expect_yielded(self, msg: &str) -> Y

Returns the contained Yielded value, consuming the self value.

Panics

Panics if the value is a Complete with a custom panic message provided by msg.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Yielded(42);
assert_eq!(x.expect_yielded("was complete"), 42);
use sans::Step;

let x: Step<i32, &str> = Step::Complete("complete");
x.expect_yielded("the world is ending"); // panics with "the world is ending"
pub fn expect_complete(self, msg: &str) -> D

Returns the contained Complete value, consuming the self value.

Panics

Panics if the value is a Yielded with a custom panic message provided by msg.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Complete("complete");
assert_eq!(x.expect_complete("was yielding"), "complete");
use sans::Step;

let x: Step<i32, &str> = Step::Yielded(42);
x.expect_complete("the world is ending"); // panics with "the world is ending"
pub fn unwrap_yielded(self) -> Y

Returns the contained Yielded value, consuming the self value.

Panics

Panics if the value is a Complete.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Yielded(42);
assert_eq!(x.unwrap_yielded(), 42);
use sans::Step;

let x: Step<i32, &str> = Step::Complete("complete");
x.unwrap_yielded(); // panics
pub fn unwrap_complete(self) -> D

Returns the contained Complete value, consuming the self value.

Panics

Panics if the value is a Yielded.

Examples
use sans::Step;

let x: Step<i32, &str> = Step::Complete("complete");
assert_eq!(x.unwrap_complete(), "complete");
use sans::Step;

let x: Step<i32, &str> = Step::Yielded(42);
x.unwrap_complete(); // panics